@@ -21,8 +21,9 @@
 import base64
 import stat
 
+import fastbencode as bencode
+
 from .. import (
-    bencode,
     errors,
     foreign,
     trace,
@@ -128,22 +129,12 @@ def fix_person_identifier(text):
 
 def decode_git_path(path):
     """Take a git path and decode it."""
-    try:
-        return path.decode('utf-8')
-    except UnicodeDecodeError:
-        if PY3:
-            return path.decode('utf-8', 'surrogateescape')
-        raise
+    return path.decode('utf-8', 'surrogateescape')
 
 
 def encode_git_path(path):
     """Take a regular path and encode it for git."""
-    try:
-        return path.encode('utf-8')
-    except UnicodeEncodeError:
-        if PY3:
-            return path.encode('utf-8', 'surrogateescape')
-        raise
+    return path.encode('utf-8', 'surrogateescape')
 
 
 def warn_escaped(commit, num_escaped):
@@ -191,7 +182,7 @@ class BzrGitMapping(foreign.VcsMapping):
         # Git paths are just bytestrings
         # We must just hope they are valid UTF-8..
         if isinstance(path, str):
-            path = path.encode("utf-8")
+            path = encode_git_path(path)
         if path == b"":
             return ROOT_ID
         return FILE_ID_PREFIX + escape_file_id(path)
@@ -319,8 +310,11 @@ class BzrGitMapping(foreign.VcsMapping):
             pass
         commit.committer = fix_person_identifier(rev.committer.encode(
             encoding))
+        first_author = rev.get_apparent_authors()[0]
+        if ',' in first_author and first_author.count('>') > 1:
+            first_author = first_author.split(',')[0]
         commit.author = fix_person_identifier(
-            rev.get_apparent_authors()[0].encode(encoding))
+            first_author.encode(encoding))
         # TODO(jelmer): Don't use this hack.
         long = getattr(__builtins__, 'long', int)
         commit.commit_time = long(rev.timestamp)
@@ -373,9 +367,11 @@ class BzrGitMapping(foreign.VcsMapping):
             i += 1
             propname = u'git-mergetag-%d' % i
         if u'git-extra' in rev.properties:
-            commit.extra.extend(
-                [l.split(b' ', 1)
-                 for l in rev.properties[u'git-extra'].splitlines()])
+            for l in rev.properties['git-extra'].splitlines():
+                (k, v) = l.split(' ', 1)
+                commit.extra.append(
+                    (k.encode('utf-8', 'surrogateescape'),
+                     v.encode('utf-8', 'surrogateescape')))
         return commit
 
     def get_revision_id(self, commit):
@@ -466,12 +462,16 @@ class BzrGitMapping(foreign.VcsMapping):
         extra_lines = []
         for k, v in commit.extra:
             if k == HG_RENAME_SOURCE:
-                extra_lines.append(k + b' ' + v + b'\n')
+                extra_lines.append(
+                    k.decode('utf-8', 'surrogateescape') + ' ' +
+                    v.decode('utf-8', 'surrogateescape') + '\n')
             elif k == HG_EXTRA:
                 hgk, hgv = v.split(b':', 1)
                 if hgk not in (HG_EXTRA_AMEND_SOURCE, ) and strict:
                     raise UnknownMercurialCommitExtra(commit, [hgk])
-                extra_lines.append(k + b' ' + v + b'\n')
+                extra_lines.append(
+                    k.decode('utf-8', 'surrogateescape') + ' ' +
+                    v.decode('utf-8', 'surrogateescape') + '\n')
             else:
                 unknown_extra_fields.append(k)
         if unknown_extra_fields and strict:
@@ -479,7 +479,7 @@ class BzrGitMapping(foreign.VcsMapping):
                 commit,
                 [f.decode('ascii', 'replace') for f in unknown_extra_fields])
         if extra_lines:
-            rev.properties[u'git-extra'] = b''.join(extra_lines)
+            rev.properties['git-extra'] = ''.join(extra_lines)
         return rev, roundtrip_revid, verifiers
 
 
@@ -488,7 +488,7 @@ class BzrGitMappingv1(BzrGitMapping):
     experimental = False
 
     def __str__(self):
-        return self.revid_prefix
+        return self.revid_prefix.decode('utf-8')
 
 
 class BzrGitMappingExperimental(BzrGitMappingv1):
